@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2003, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2003, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,23 +25,23 @@
 
 package sun.io;
 
-import sun.nio.cs.ISO_8859_13;
+import sun.nio.cs.ext.ISO_8859_3;
 
 /**
- * A table to convert ISO8859_13 to Unicode
+ * A table to convert ISO8859_3 to Unicode
  *
  * @author  ConverterGenerator tool
  */
 
-public class ByteToCharISO8859_13 extends ByteToCharSingleByte {
+public class ByteToCharISO8859_3 extends ByteToCharSingleByte {
 
-    private final static ISO_8859_13 nioCoder = new ISO_8859_13();
+    private final static ISO_8859_3 nioCoder = new ISO_8859_3();
 
     public String getCharacterEncoding() {
-        return "ISO8859_13";
+        return "ISO8859_3";
     }
 
-    public ByteToCharISO8859_13() {
+    public ByteToCharISO8859_3() {
         super.byteToCharTable = nioCoder.getDecoderSingleByteMappings();
     }
 }
