@@ -45,7 +45,7 @@ final class AixPollPort
     private static final Unsafe unsafe = Unsafe.getUnsafe();
 
     static {
-        Util.load();
+        IOUtil.load();
         init();
     }
 
